@@ -46,6 +46,7 @@ class JobWidget(urwid.WidgetWrap):
         self.columns = OrderedDict()
         self.columns["selected"] = urwid.Text("", wrap="ellipsis")
         self.columns["job_id"] = urwid.Text("", wrap="ellipsis")
+        self.columns["array"] = urwid.Text("", wrap="ellipsis")
         self.columns["user"] = urwid.Text("", wrap="ellipsis")
         self.columns["name"] = urwid.Text("", wrap="ellipsis")
         self.columns["state"] = urwid.AttrMap(urwid.Text("", wrap="ellipsis"), None)
@@ -73,6 +74,7 @@ class JobWidget(urwid.WidgetWrap):
 
     def update_values(self, job):
         self.columns["job_id"].set_text(job.job_id)
+        self.columns["array"].set_text(job.array_str())
         self.columns["user"].set_text(job.user)
         self.columns["name"].set_text(job.name)
         self.columns["state"]._original_widget.set_text(job.state.title())
@@ -118,6 +120,7 @@ class JobQueueWidget(urwid.WidgetWrap):
     column_widths = [
         (2,),
         (10,),
+        (10,),
         ("weight", 1),
         ("weight", 2),
         (14,),
@@ -133,6 +136,7 @@ class JobQueueWidget(urwid.WidgetWrap):
         column_labels = [
             "",
             "Job ID",
+            "Job Array",
             "User",
             "Name",
             "State",
@@ -603,7 +607,7 @@ class StuiWidget(urwid.WidgetWrap):
         self.cluster = cluster
 
         self.header_time = urwid.Text(datetime.now().strftime("%X"), align="right")
-        self.header_cluster_name = urwid.Text("Cluster: N/A")
+        self.header_cluster_name = urwid.Text("Cluster: N/A", align="center")
 
         header = urwid.Columns(
             [
@@ -633,7 +637,7 @@ class StuiWidget(urwid.WidgetWrap):
 
     def set_cluster_name(self, cluster_name):
         self.header_cluster_name.set_text(
-            [(None, "Cluster: "), ("magenta", cluster_name)]
+            [(None, "Cluster:"), ("magenta", cluster_name)]
         )
 
     def update_time(self):
@@ -658,9 +662,34 @@ class StuiWidget(urwid.WidgetWrap):
 
         self.view_placeholder.original_widget = overlay
 
+    def password_prompt_popup(self, ok_handler, cancel_handler):
+
+        w = widgets.PasswordPrompt(
+            cancel_handler,
+            title="SSH Connection Failed",
+            msg="No authentication methods (SSH keys/agent) found. Enter login details manually:",
+        )
+
+        urwid.connect_signal(w, "user_password_provided", ok_handler)
+
+        overlay = urwid.Overlay(
+            w,
+            self.view,
+            align="center",
+            width=("relative", 40),
+            valign=("relative", 30),
+            height="pack",
+        )
+
+        self.view_placeholder.original_widget = overlay
+
     def cluster_connected_callback(self):
         self.set_cluster_name(self.cluster.get_name())
         self.refresh_jobs()
+        self.close_popup()
+
+    def close_popup(self, *args, **kwargs):
+        # TODO: Assert that there's an active popup
         self.view_placeholder.original_widget = self.view
 
 
@@ -697,16 +726,33 @@ class StuiApp(object):
             self.w, self.palette, unhandled_input=self.exit_on_q,
         )
 
-        fd = self.loop.watch_pipe(self.cluster_connected_callback)
-        self.cluster.connect(fd)
+        self.fd = self.loop.watch_pipe(self.cluster_connect_handler)
+        self.cluster.connect(self.fd)
 
-    def cluster_connected_callback(self, foo):
-        self.register_refresh()
-        self.w.cluster_connected_callback()
+    def ssh_login_provided_callback(self, user, password):
+        self.w.connecting_popup()
+        self.cluster.connect(self.fd, user, password)
 
-        # Return False so that the watch is removed from main loop and its read-end of
-        # the pipe is closed. The write-end of the pipe will be closed on the backend.
-        return False
+    def cluster_connect_handler(self, message):
+        if message == b"need password" or message == b"wrong password":
+
+            def exit(*args, **kwargs):
+                raise urwid.ExitMainLoop()
+
+            self.w.password_prompt_popup(
+                ok_handler=self.ssh_login_provided_callback, cancel_handler=exit
+            )
+            return True
+
+        elif message == b"connection established":
+            self.fd = None  # TODO: delattr?
+            self.register_refresh()
+            self.w.cluster_connected_callback()
+
+            # Return False so that the watch is removed from main loop and its read-end
+            # of the pipe is closed. The write-end of the pipe will be closed on the
+            # backend.
+            return False
 
     def run(self):
         # self.loop.screen.set_terminal_properties(bright_is_bold=False)
@@ -748,6 +794,14 @@ def parse_args():
         help="Remote destination where slurm controller is running. Format: --remote {Host name defined in ssh config} or --remote {username@server}. Does _not_ prompt for password and relies on ssh-keys for authentication.",
     )
 
+    parser.add_argument(
+        "-r",
+        "--refresh-interval",
+        type=int,
+        default=1,
+        help="Refresh interval (in seconds) for fetching data from the cluster. (Default: 1s)",
+    )
+
     args = parser.parse_args()
 
     return args
